#include "file_services.h"

int dir_entries_count;

char dir_entries[MAX_ENTRIES][18];

int init_fs(int do_pin, int clk_pin, int di_pin, int cs_pin)
{
    int ret;
    
    ret = sd_mount(do_pin, clk_pin, di_pin, cs_pin);
    
    return ret;
}

int create_dir(char *path)
{
    int ret;
    
    ret = mkdir(path, 0);
    
    return ret;
}    

int delete_dir(char *path)
{
    int ret;
    
    ret = rmdir(path);
    
    return ret;
}

FILE *open_file(char *filename, char *mode)
{
    FILE* file_pointer = fopen(filename, mode);
    
    return file_pointer;
}

int close_file(FILE *file_pointer)
{
    int ret;

    ret = fclose(file_pointer);

    return ret;
}

size_t write_file(void *ptr, size_t size, size_t count, FILE *file_pointer)
{
    size_t bytes_written;
    
    bytes_written = fwrite(ptr, size, count, file_pointer);
    
    return bytes_written;
}    

int delete_file(char *filename)
{
    int ret;
    
    ret = remove(filename);
    
    return ret;
}       

char *get_curr_dir(char *buf, int size)
{
    getcwd(buf, size);
    
    return buf;
}    

int set_curr_dir(char *path)
{
    int ret;

    ret = chdir(path);
    
    return ret;
}

char *path_add_slash(char *path)
{
    int path_length;
    char slash = '/';

    if(path)
    {
        path_length = strlen(path);

        if(path[path_length - 1] == slash)
        {
            return &path[path_length];
        }            
        else
        {
            path[path_length] = slash;
            path[path_length + 1] = '\0';

            return &path[path_length + 1];
        }
    }

    return NULL;
}

bool path_append(char *path, char *more)
{
    int path_length;
    char slash = '/';

    if(path && more)
    {
        path_length = strlen(path);

        if(path[path_length - 1] != slash)
        {
            path_add_slash(path);
        }            

        strcat(path, more);
    }

    return false;
}

void sort()
{
    bool incomplete = true;
    
    while(incomplete)
    {
        incomplete = false;

        for(int pos = 0; pos < dir_entries_count; pos++)
        {
            incomplete |= compare_swap(pos);
        }
    }
}

bool compare_swap(int pos)
{
    char temp[13];
      
    int pos_first = pos;   
    int pos_next = pos + 1;

    if(strcmp(dir_entries[pos_first], dir_entries[pos_next]) > 0)  
    {
        strcpy(temp, dir_entries[pos_first]);
        memset(dir_entries[pos_first], 0, sizeof(dir_entries[pos_first]));      
        strcpy(dir_entries[pos_first], dir_entries[pos_next]);
        memset(dir_entries[pos_next], 0, sizeof(dir_entries[pos_next]));
        strcpy(dir_entries[pos_next], temp);      

        return true;   
    }   
   
    return false;
}

int index_dir_entries(char *dir_path)
{
    dir_entries_count = -1;

    DIR *sd_dir;
    struct dirent *dir_entry;

    sd_dir = opendir(dir_path);

    if(sd_dir != NULL)
    {
        while((dir_entry = readdir(sd_dir)))
        {
            dir_entries_count++;
            
            if(dir_entry->attr == ATTR_DIRECTORY)
            {
                strcpy(dir_entries[dir_entries_count], "(DIR)");
                strcat(dir_entries[dir_entries_count], dir_entry->d_name);
            }
            else
            {
                strcpy(dir_entries[dir_entries_count], dir_entry->d_name);
            }                
        }        

        closedir(sd_dir);
        
        sort();
    }
    else
    {
        print("Couldn't open the directory");
    }        

    return 0;
}

void print_dir_entries()
{
    for(int i = 0; i != dir_entries_count + 1; i++)
    {
        print("%s\n", dir_entries[i]);
    }
}